@@ -1,19 +1,39 @@
 """Support for deCONZ sensors."""
 from pydeconz.sensor import (
+    AncillaryControl,
     Battery,
     Consumption,
     Daylight,
+    DoorLock,
+    Humidity,
     LightLevel,
     Power,
+    Pressure,
     Switch,
+    Temperature,
     Thermostat,
 )
 
+from homeassistant.components.sensor import (
+    DOMAIN,
+    STATE_CLASS_MEASUREMENT,
+    SensorEntity,
+)
 from homeassistant.const import (
     ATTR_TEMPERATURE,
     ATTR_VOLTAGE,
     DEVICE_CLASS_BATTERY,
-    UNIT_PERCENTAGE,
+    DEVICE_CLASS_HUMIDITY,
+    DEVICE_CLASS_ILLUMINANCE,
+    DEVICE_CLASS_POWER,
+    DEVICE_CLASS_PRESSURE,
+    DEVICE_CLASS_TEMPERATURE,
+    ENERGY_KILO_WATT_HOUR,
+    LIGHT_LUX,
+    PERCENTAGE,
+    POWER_WATT,
+    PRESSURE_HPA,
+    TEMP_CELSIUS,
 )
 from homeassistant.core import callback
 from homeassistant.helpers.dispatcher import (
@@ -23,7 +43,6 @@ from homeassistant.helpers.dispatcher import (
 
 from .const import ATTR_DARK, ATTR_ON, NEW_SENSOR
 from .deconz_device import DeconzDevice
-from .deconz_event import DeconzEvent
 from .gateway import get_gateway_from_config_entry
 
 ATTR_CURRENT = "current"
@@ -31,63 +50,90 @@ ATTR_POWER = "power"
 ATTR_DAYLIGHT = "daylight"
 ATTR_EVENT_ID = "event_id"
 
+DEVICE_CLASS = {
+    Humidity: DEVICE_CLASS_HUMIDITY,
+    LightLevel: DEVICE_CLASS_ILLUMINANCE,
+    Power: DEVICE_CLASS_POWER,
+    Pressure: DEVICE_CLASS_PRESSURE,
+    Temperature: DEVICE_CLASS_TEMPERATURE,
+}
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
-    """Old way of setting up deCONZ platforms."""
+ICON = {
+    Daylight: "mdi:white-balance-sunny",
+    Pressure: "mdi:gauge",
+    Temperature: "mdi:thermometer",
+}
+
+STATE_CLASS = {
+    Humidity: STATE_CLASS_MEASUREMENT,
+    Pressure: STATE_CLASS_MEASUREMENT,
+    Temperature: STATE_CLASS_MEASUREMENT,
+}
+
+UNIT_OF_MEASUREMENT = {
+    Consumption: ENERGY_KILO_WATT_HOUR,
+    Humidity: PERCENTAGE,
+    LightLevel: LIGHT_LUX,
+    Power: POWER_WATT,
+    Pressure: PRESSURE_HPA,
+    Temperature: TEMP_CELSIUS,
+}
 
 
 async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up the deCONZ sensors."""
     gateway = get_gateway_from_config_entry(hass, config_entry)
+    gateway.entities[DOMAIN] = set()
 
-    batteries = set()
     battery_handler = DeconzBatteryHandler(gateway)
 
     @callback
-    def async_add_sensor(sensors, new=True):
+    def async_add_sensor(sensors=gateway.api.sensors.values()):
         """Add sensors from deCONZ.
 
-        Create DeconzEvent if part of ZHAType list.
-        Create DeconzSensor if not a ZHAType and not a binary sensor.
         Create DeconzBattery if sensor has a battery attribute.
-        If new is false it means an existing sensor has got a battery state reported.
+        Create DeconzSensor if not a battery, switch or thermostat and not a binary sensor.
         """
         entities = []
 
         for sensor in sensors:
 
-            if new and sensor.type in Switch.ZHATYPE:
-
-                if gateway.option_allow_clip_sensor or not sensor.type.startswith(
-                    "CLIP"
-                ):
-                    new_event = DeconzEvent(sensor, gateway)
-                    hass.async_create_task(new_event.async_update_device_registry())
-                    gateway.events.append(new_event)
-
-            elif (
-                new
-                and sensor.BINARY is False
-                and sensor.type not in Battery.ZHATYPE + Thermostat.ZHATYPE
-                and (
-                    gateway.option_allow_clip_sensor
-                    or not sensor.type.startswith("CLIP")
-                )
-            ):
-                entities.append(DeconzSensor(sensor, gateway))
+            if not gateway.option_allow_clip_sensor and sensor.type.startswith("CLIP"):
+                continue
 
             if sensor.battery is not None:
+                battery_handler.remove_tracker(sensor)
+
+                known_batteries = set(gateway.entities[DOMAIN])
                 new_battery = DeconzBattery(sensor, gateway)
-                if new_battery.unique_id not in batteries:
-                    batteries.add(new_battery.unique_id)
+                if new_battery.unique_id not in known_batteries:
                     entities.append(new_battery)
-                    battery_handler.remove_tracker(sensor)
+
             else:
                 battery_handler.create_tracker(sensor)
 
-        async_add_entities(entities, True)
+            if (
+                not sensor.BINARY
+                and sensor.type
+                not in AncillaryControl.ZHATYPE
+                + Battery.ZHATYPE
+                + DoorLock.ZHATYPE
+                + Switch.ZHATYPE
+                + Thermostat.ZHATYPE
+                and sensor.uniqueid not in gateway.entities[DOMAIN]
+            ):
+                entities.append(DeconzSensor(sensor, gateway))
 
-    gateway.listeners.append(
+            if sensor.secondary_temperature:
+                known_temperature_sensors = set(gateway.entities[DOMAIN])
+                new_temperature_sensor = DeconzTemperature(sensor, gateway)
+                if new_temperature_sensor.unique_id not in known_temperature_sensors:
+                    entities.append(new_temperature_sensor)
+
+        if entities:
+            async_add_entities(entities)
+
+    config_entry.async_on_unload(
         async_dispatcher_connect(
             hass, gateway.async_signal_new_device(NEW_SENSOR), async_add_sensor
         )
@@ -98,18 +144,26 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     )
 
 
-class DeconzSensor(DeconzDevice):
+class DeconzSensor(DeconzDevice, SensorEntity):
     """Representation of a deCONZ sensor."""
 
-    @callback
-    def async_update_callback(self, force_update=False, ignore_update=False):
-        """Update the sensor's state."""
-        if ignore_update:
-            return
+    TYPE = DOMAIN
 
+    def __init__(self, device, gateway):
+        """Initialize deCONZ binary sensor."""
+        super().__init__(device, gateway)
+
+        self._attr_device_class = DEVICE_CLASS.get(type(self._device))
+        self._attr_icon = ICON.get(type(self._device))
+        self._attr_state_class = STATE_CLASS.get(type(self._device))
+        self._attr_unit_of_measurement = UNIT_OF_MEASUREMENT.get(type(self._device))
+
+    @callback
+    def async_update_callback(self, force_update=False):
+        """Update the sensor's state."""
         keys = {"on", "reachable", "state"}
         if force_update or self._device.changed_keys.intersection(keys):
-            self.async_write_ha_state()
+            super().async_update_callback(force_update=force_update)
 
     @property
     def state(self):
@@ -117,22 +171,7 @@ class DeconzSensor(DeconzDevice):
         return self._device.state
 
     @property
-    def device_class(self):
-        """Return the class of the sensor."""
-        return self._device.SENSOR_CLASS
-
-    @property
-    def icon(self):
-        """Return the icon to use in the frontend."""
-        return self._device.SENSOR_ICON
-
-    @property
-    def unit_of_measurement(self):
-        """Return the unit of measurement of this sensor."""
-        return self._device.SENSOR_UNIT
-
-    @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes of the sensor."""
         attr = {}
 
@@ -163,22 +202,78 @@ class DeconzSensor(DeconzDevice):
         return attr
 
 
-class DeconzBattery(DeconzDevice):
-    """Battery class for when a device is only represented as an event."""
+class DeconzTemperature(DeconzDevice, SensorEntity):
+    """Representation of a deCONZ temperature sensor.
 
-    @callback
-    def async_update_callback(self, force_update=False, ignore_update=False):
-        """Update the battery's state, if needed."""
-        if ignore_update:
-            return
+    Extra temperature sensor on certain Xiaomi devices.
+    """
 
-        keys = {"battery", "reachable"}
-        if force_update or self._device.changed_keys.intersection(keys):
-            self.async_write_ha_state()
+    _attr_device_class = DEVICE_CLASS_TEMPERATURE
+    _attr_state_class = STATE_CLASS_MEASUREMENT
+    _attr_unit_of_measurement = TEMP_CELSIUS
+
+    TYPE = DOMAIN
+
+    def __init__(self, device, gateway):
+        """Initialize deCONZ temperature sensor."""
+        super().__init__(device, gateway)
+
+        self._attr_name = f"{self._device.name} Temperature"
 
     @property
     def unique_id(self):
         """Return a unique identifier for this device."""
+        return f"{self.serial}-temperature"
+
+    @callback
+    def async_update_callback(self, force_update=False):
+        """Update the sensor's state."""
+        keys = {"temperature", "reachable"}
+        if force_update or self._device.changed_keys.intersection(keys):
+            super().async_update_callback(force_update=force_update)
+
+    @property
+    def state(self):
+        """Return the state of the sensor."""
+        return self._device.secondary_temperature
+
+
+class DeconzBattery(DeconzDevice, SensorEntity):
+    """Battery class for when a device is only represented as an event."""
+
+    _attr_device_class = DEVICE_CLASS_BATTERY
+    _attr_state_class = STATE_CLASS_MEASUREMENT
+    _attr_unit_of_measurement = PERCENTAGE
+
+    TYPE = DOMAIN
+
+    def __init__(self, device, gateway):
+        """Initialize deCONZ battery level sensor."""
+        super().__init__(device, gateway)
+
+        self._attr_name = f"{self._device.name} Battery Level"
+
+    @callback
+    def async_update_callback(self, force_update=False):
+        """Update the battery's state, if needed."""
+        keys = {"battery", "reachable"}
+        if force_update or self._device.changed_keys.intersection(keys):
+            super().async_update_callback(force_update=force_update)
+
+    @property
+    def unique_id(self):
+        """Return a unique identifier for this device.
+
+        Normally there should only be one battery sensor per device from deCONZ.
+        With specific Danfoss devices each endpoint can report its own battery state.
+        """
+        if self._device.manufacturer == "Danfoss" and self._device.modelid in [
+            "0x8030",
+            "0x8031",
+            "0x8034",
+            "0x8035",
+        ]:
+            return f"{super().unique_id}-battery"
         return f"{self.serial}-battery"
 
     @property
@@ -187,22 +282,7 @@ class DeconzBattery(DeconzDevice):
         return self._device.battery
 
     @property
-    def name(self):
-        """Return the name of the battery."""
-        return f"{self._device.name} Battery Level"
-
-    @property
-    def device_class(self):
-        """Return the class of the sensor."""
-        return DEVICE_CLASS_BATTERY
-
-    @property
-    def unit_of_measurement(self):
-        """Return the unit of measurement of this entity."""
-        return UNIT_PERCENTAGE
-
-    @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes of the battery."""
         attr = {}
 
@@ -238,7 +318,6 @@ class DeconzSensorStateTracker:
                 self.gateway.hass,
                 self.gateway.async_signal_new_device(NEW_SENSOR),
                 [self.sensor],
-                False,
             )
 
 
