@@ -59,6 +59,7 @@ CONF_PERCENTAGE_STATE_TOPIC = "percentage_state_topic"
 CONF_PERCENTAGE_COMMAND_TOPIC = "percentage_command_topic"
 CONF_PERCENTAGE_VALUE_TEMPLATE = "percentage_value_template"
 CONF_PERCENTAGE_COMMAND_TEMPLATE = "percentage_command_template"
+CONF_PAYLOAD_RESET_PERCENTAGE = "payload_reset_percentage"
 CONF_SPEED_RANGE_MIN = "speed_range_min"
 CONF_SPEED_RANGE_MAX = "speed_range_max"
 CONF_PRESET_MODE_STATE_TOPIC = "preset_mode_state_topic"
@@ -66,6 +67,7 @@ CONF_PRESET_MODE_COMMAND_TOPIC = "preset_mode_command_topic"
 CONF_PRESET_MODE_VALUE_TEMPLATE = "preset_mode_value_template"
 CONF_PRESET_MODE_COMMAND_TEMPLATE = "preset_mode_command_template"
 CONF_PRESET_MODES_LIST = "preset_modes"
+CONF_PAYLOAD_RESET_PRESET_MODE = "payload_reset_preset_mode"
 CONF_SPEED_STATE_TOPIC = "speed_state_topic"
 CONF_SPEED_COMMAND_TOPIC = "speed_command_topic"
 CONF_SPEED_VALUE_TEMPLATE = "speed_value_template"
@@ -84,6 +86,7 @@ CONF_SPEED_LIST = "speeds"
 DEFAULT_NAME = "MQTT Fan"
 DEFAULT_PAYLOAD_ON = "ON"
 DEFAULT_PAYLOAD_OFF = "OFF"
+DEFAULT_PAYLOAD_RESET = "None"
 DEFAULT_OPTIMISTIC = False
 DEFAULT_SPEED_RANGE_MIN = 1
 DEFAULT_SPEED_RANGE_MAX = 100
@@ -91,6 +94,18 @@ DEFAULT_SPEED_RANGE_MAX = 100
 OSCILLATE_ON_PAYLOAD = "oscillate_on"
 OSCILLATE_OFF_PAYLOAD = "oscillate_off"
 
+MQTT_FAN_ATTRIBUTES_BLOCKED = frozenset(
+    {
+        fan.ATTR_DIRECTION,
+        fan.ATTR_OSCILLATING,
+        fan.ATTR_PERCENTAGE_STEP,
+        fan.ATTR_PERCENTAGE,
+        fan.ATTR_PRESET_MODE,
+        fan.ATTR_PRESET_MODES,
+        fan.ATTR_SPEED_LIST,
+        fan.ATTR_SPEED,
+    }
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -113,15 +128,22 @@ def valid_speed_range_configuration(config):
     return config
 
 
+def valid_preset_mode_configuration(config):
+    """Validate that the preset mode reset payload is not one of the preset modes."""
+    if config.get(CONF_PAYLOAD_RESET_PRESET_MODE) in config.get(CONF_PRESET_MODES_LIST):
+        raise ValueError("preset_modes must not contain payload_reset_preset_mode")
+    return config
+
+
 PLATFORM_SCHEMA = vol.All(
-    # CONF_SPEED_COMMAND_TOPIC, CONF_SPEED_STATE_TOPIC, CONF_STATE_VALUE_TEMPLATE, CONF_SPEED_LIST and
+    # CONF_SPEED_COMMAND_TOPIC, CONF_SPEED_LIST, CONF_SPEED_STATE_TOPIC, CONF_SPEED_VALUE_TEMPLATE and
     # Speeds SPEED_LOW, SPEED_MEDIUM, SPEED_HIGH SPEED_OFF,
     # are deprecated, support will be removed after a quarter (2021.7)
     cv.deprecated(CONF_PAYLOAD_HIGH_SPEED),
     cv.deprecated(CONF_PAYLOAD_LOW_SPEED),
     cv.deprecated(CONF_PAYLOAD_MEDIUM_SPEED),
-    cv.deprecated(CONF_SPEED_LIST),
     cv.deprecated(CONF_SPEED_COMMAND_TOPIC),
+    cv.deprecated(CONF_SPEED_LIST),
     cv.deprecated(CONF_SPEED_STATE_TOPIC),
     cv.deprecated(CONF_SPEED_VALUE_TEMPLATE),
     mqtt.MQTT_RW_PLATFORM_SCHEMA.extend(
@@ -153,6 +175,12 @@ PLATFORM_SCHEMA = vol.All(
             vol.Optional(
                 CONF_SPEED_RANGE_MAX, default=DEFAULT_SPEED_RANGE_MAX
             ): cv.positive_int,
+            vol.Optional(
+                CONF_PAYLOAD_RESET_PERCENTAGE, default=DEFAULT_PAYLOAD_RESET
+            ): cv.string,
+            vol.Optional(
+                CONF_PAYLOAD_RESET_PRESET_MODE, default=DEFAULT_PAYLOAD_RESET
+            ): cv.string,
             vol.Optional(CONF_PAYLOAD_HIGH_SPEED, default=SPEED_HIGH): cv.string,
             vol.Optional(CONF_PAYLOAD_LOW_SPEED, default=SPEED_LOW): cv.string,
             vol.Optional(CONF_PAYLOAD_MEDIUM_SPEED, default=SPEED_MEDIUM): cv.string,
@@ -177,6 +205,7 @@ PLATFORM_SCHEMA = vol.All(
     ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema),
     valid_fan_speed_configuration,
     valid_speed_range_configuration,
+    valid_preset_mode_configuration,
 )
 
 
@@ -206,6 +235,8 @@ async def _async_setup_entity(
 class MqttFan(MqttEntity, FanEntity):
     """A MQTT fan component."""
 
+    _attributes_extra_blocked = MQTT_FAN_ATTRIBUTES_BLOCKED
+
     def __init__(self, hass, config, config_entry, discovery_data):
         """Initialize the MQTT fan."""
         self._state = False
@@ -281,6 +312,8 @@ class MqttFan(MqttEntity, FanEntity):
             "SPEED_MEDIUM": config[CONF_PAYLOAD_MEDIUM_SPEED],
             "SPEED_HIGH": config[CONF_PAYLOAD_HIGH_SPEED],
             "SPEED_OFF": config[CONF_PAYLOAD_OFF_SPEED],
+            "PERCENTAGE_RESET": config[CONF_PAYLOAD_RESET_PERCENTAGE],
+            "PRESET_MODE_RESET": config[CONF_PAYLOAD_RESET_PRESET_MODE],
         }
         # The use of legacy speeds is deprecated in the schema, support will be removed after a quarter (2021.7)
         self._feature_legacy_speeds = not self._topic[CONF_SPEED_COMMAND_TOPIC] is None
@@ -364,20 +397,27 @@ class MqttFan(MqttEntity, FanEntity):
         @log_messages(self.hass, self.entity_id)
         def percentage_received(msg):
             """Handle new received MQTT message for the percentage."""
-            numeric_val_str = self._value_templates[ATTR_PERCENTAGE](msg.payload)
-            if not numeric_val_str:
+            rendered_percentage_payload = self._value_templates[ATTR_PERCENTAGE](
+                msg.payload
+            )
+            if not rendered_percentage_payload:
                 _LOGGER.debug("Ignoring empty speed from '%s'", msg.topic)
                 return
+            if rendered_percentage_payload == self._payload["PERCENTAGE_RESET"]:
+                self._percentage = None
+                self._speed = None
+                self.async_write_ha_state()
+                return
             try:
                 percentage = ranged_value_to_percentage(
-                    self._speed_range, int(numeric_val_str)
+                    self._speed_range, int(rendered_percentage_payload)
                 )
             except ValueError:
                 _LOGGER.warning(
                     "'%s' received on topic %s. '%s' is not a valid speed within the speed range",
                     msg.payload,
                     msg.topic,
-                    numeric_val_str,
+                    rendered_percentage_payload,
                 )
                 return
             if percentage < 0 or percentage > 100:
@@ -385,7 +425,7 @@ class MqttFan(MqttEntity, FanEntity):
                     "'%s' received on topic %s. '%s' is not a valid speed within the speed range",
                     msg.payload,
                     msg.topic,
-                    numeric_val_str,
+                    rendered_percentage_payload,
                 )
                 return
             self._percentage = percentage
@@ -404,6 +444,10 @@ class MqttFan(MqttEntity, FanEntity):
         def preset_mode_received(msg):
             """Handle new received MQTT message for preset mode."""
             preset_mode = self._value_templates[ATTR_PRESET_MODE](msg.payload)
+            if preset_mode == self._payload["PRESET_MODE_RESET"]:
+                self._preset_mode = None
+                self.async_write_ha_state()
+                return
             if not preset_mode:
                 _LOGGER.debug("Ignoring empty preset_mode from '%s'", msg.topic)
                 return
@@ -658,7 +702,7 @@ class MqttFan(MqttEntity, FanEntity):
 
         if self._optimistic_preset_mode:
             self._preset_mode = preset_mode
-        self.async_write_ha_state()
+            self.async_write_ha_state()
 
     # async_set_speed is deprecated, support will be removed after a quarter (2021.7)
     async def async_set_speed(self, speed: str) -> None:
@@ -691,7 +735,7 @@ class MqttFan(MqttEntity, FanEntity):
 
             if self._optimistic_speed and speed_payload:
                 self._speed = speed
-            self.async_write_ha_state()
+                self.async_write_ha_state()
 
     async def async_oscillate(self, oscillating: bool) -> None:
         """Set oscillation.
