@@ -1,18 +1,17 @@
 """Support for Velbus devices."""
-import asyncio
 import logging
 
 import velbus
 import voluptuous as vol
 
 from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
-from homeassistant.const import CONF_NAME, CONF_PORT
+from homeassistant.const import CONF_ADDRESS, CONF_NAME, CONF_PORT
+from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity import Entity
-from homeassistant.helpers.typing import HomeAssistantType
 
-from .const import DOMAIN
+from .const import CONF_MEMO_TEXT, DOMAIN, SERVICE_SET_MEMO_TEXT
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -22,7 +21,7 @@ CONFIG_SCHEMA = vol.Schema(
     {DOMAIN: vol.Schema({vol.Required(CONF_PORT): cv.string})}, extra=vol.ALLOW_EXTRA
 )
 
-COMPONENT_TYPES = ["switch", "sensor", "binary_sensor", "cover", "climate", "light"]
+PLATFORMS = ["switch", "sensor", "binary_sensor", "cover", "climate", "light"]
 
 
 async def async_setup(hass, config):
@@ -44,33 +43,33 @@ async def async_setup(hass, config):
     return True
 
 
-async def async_setup_entry(hass: HomeAssistantType, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Establish connection with velbus."""
     hass.data.setdefault(DOMAIN, {})
 
     def callback():
         modules = controller.get_modules()
         discovery_info = {"cntrl": controller}
-        for category in COMPONENT_TYPES:
-            discovery_info[category] = []
+        for platform in PLATFORMS:
+            discovery_info[platform] = []
         for module in modules:
             for channel in range(1, module.number_of_channels() + 1):
-                for category in COMPONENT_TYPES:
-                    if category in module.get_categories(channel):
-                        discovery_info[category].append(
+                for platform in PLATFORMS:
+                    if platform in module.get_categories(channel):
+                        discovery_info[platform].append(
                             (module.get_module_address(), channel)
                         )
         hass.data[DOMAIN][entry.entry_id] = discovery_info
 
-        for category in COMPONENT_TYPES:
-            hass.add_job(hass.config_entries.async_forward_entry_setup(entry, category))
+        for platform in PLATFORMS:
+            hass.add_job(hass.config_entries.async_forward_entry_setup(entry, platform))
 
     try:
         controller = velbus.Controller(entry.data[CONF_PORT])
         controller.scan(callback)
     except velbus.util.VelbusException as err:
         _LOGGER.error("An error occurred: %s", err)
-        raise ConfigEntryNotReady
+        raise ConfigEntryNotReady from err
 
     def syn_clock(self, service=None):
         try:
@@ -80,22 +79,43 @@ async def async_setup_entry(hass: HomeAssistantType, entry: ConfigEntry):
 
     hass.services.async_register(DOMAIN, "sync_clock", syn_clock, schema=vol.Schema({}))
 
+    def set_memo_text(service):
+        """Handle Memo Text service call."""
+        module_address = service.data[CONF_ADDRESS]
+        memo_text = service.data[CONF_MEMO_TEXT]
+        memo_text.hass = hass
+        try:
+            controller.get_module(module_address).set_memo_text(
+                memo_text.async_render()
+            )
+        except velbus.util.VelbusException as err:
+            _LOGGER.error("An error occurred while setting memo text: %s", err)
+
+    hass.services.async_register(
+        DOMAIN,
+        SERVICE_SET_MEMO_TEXT,
+        set_memo_text,
+        vol.Schema(
+            {
+                vol.Required(CONF_ADDRESS): vol.All(
+                    vol.Coerce(int), vol.Range(min=0, max=255)
+                ),
+                vol.Optional(CONF_MEMO_TEXT, default=""): cv.template,
+            }
+        ),
+    )
+
     return True
 
 
-async def async_unload_entry(hass: HomeAssistantType, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Remove the velbus connection."""
-    await asyncio.wait(
-        [
-            hass.config_entries.async_forward_entry_unload(entry, component)
-            for component in COMPONENT_TYPES
-        ]
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     hass.data[DOMAIN][entry.entry_id]["cntrl"].stop()
     hass.data[DOMAIN].pop(entry.entry_id)
     if not hass.data[DOMAIN]:
         hass.data.pop(DOMAIN)
-    return True
+    return unload_ok
 
 
 class VelbusEntity(Entity):
