@@ -13,7 +13,6 @@ from script.hassfest.model import Integration
 
 COMMENT_REQUIREMENTS = (
     "Adafruit_BBIO",
-    "Adafruit-DHT",
     "avea",  # depends on bluepy
     "avion",
     "beacontools",
@@ -28,7 +27,6 @@ COMMENT_REQUIREMENTS = (
     "face_recognition",
     "i2csense",
     "opencv-python-headless",
-    "py_noaa",
     "pybluez",
     "pycups",
     "PySwitchbot",
@@ -66,10 +64,6 @@ urllib3>=1.24.3
 # Constrain H11 to ensure we get a new enough version to support non-rfc line endings
 h11>=0.12.0
 
-# Constrain httpcore to fix exception when connection dropped
-# https://github.com/encode/httpcore/issues/239
-httpcore>=0.12.3
-
 # Constrain httplib2 to protect against GHSA-93xj-8mrv-444m
 # https://github.com/advisories/GHSA-93xj-8mrv-444m
 httplib2>=0.19.0
@@ -89,6 +83,10 @@ enum34==1000000000.0.0
 typing==1000000000.0.0
 uuid==1000000000.0.0
 
+# httpcore 0.13.4 breaks several integrations
+# https://github.com/home-assistant/core/issues/51778
+httpcore==0.13.3
+
 """
 
 IGNORE_PRE_COMMIT_HOOK_ID = (
@@ -96,6 +94,7 @@ IGNORE_PRE_COMMIT_HOOK_ID = (
     "check-json",
     "no-commit-to-branch",
     "prettier",
+    "python-typing-update",
 )
 
 
